import { Component } from '@angular/core';
import { User } from '../../../models/user.model';
import { AdminService } from '../../services/admin.service';
import { AdministredStructuresComponent } from './administred-structures/administred-structures.component';
import { JobRenderer } from './job-renderer/job-renderer.component';
import { Job } from '../../../models/job.model';
import { Employer } from '../../../models/employer.model';
import { EmployerRenderer } from './employer-renderer/employer-renderer.component';
import { NotificationService } from '../../../services/notification.service';
import { DeleteUserComponent } from './delete-user/delete-user.component';

@Component({
  selector: 'app-admin-manage-users',
  templateUrl: './manage-users.component.html',
  styleUrls: ['./manage-users.component.scss'],
})
export class ManageUsersComponent {
  public attachedUsers: User[] = [];
  public unAttachedUsers: User[] = [];
  public unVerifiedUsers: User[] = [];
  public validatedJobsName: string[] = [];
  public validatedJobs: Job[] = [];
  public validatedEmployersName: string[] = [];
  public validatedEmployers: Employer[] = [];
  public deleteModalOpenned = false;
  public editJobModalOpenned = false;
  public editEmployerModalOpenned = false;
  public userToDelete: User = null;
  public changingJobs: Job[] = [
    { name: null, hasPersonalOffer: true, validated: false },
    { name: null, hasPersonalOffer: true, validated: false },
  ];
  public changingEmployers: Employer[] = [
    { name: null, validated: false },
    { name: null, validated: false },
  ];
  public contextRow: any;

  public columnDefs;
  public frameworkComponents;
  public defaultColDef = {
    editable: true,
    sortable: true,
    flex: 1,
    minWidth: 100,
    filter: true,
    resizable: true,
  };
  public unAttachedColDef = {
    ...this.defaultColDef,
    cellStyle: {
      color: '#da6c2e',
    },
  };

  constructor(private adminService: AdminService, private notificationService: NotificationService) {
    this.findValidatedJobs();
    this.findValidatedEmployers();
    this.columnDefs = [
      {
        headerName: 'Nom',
        field: 'surname',
      },
      {
        headerName: 'Prénom',
        field: 'name',
      },
      {
        minWidth: 150,
        headerName: 'Mail',
        field: 'email',
      },
      {
        minWidth: 100,
        headerName: 'Téléphone',
        field: 'phone',
      },
      {
        headerName: 'Structures administrées',
        cellRenderer: 'administredStructuresComponent',
        cellRendererParams: {
          structures: 'structures',
        },
        minWidth: 350,
      },
      {
        headerName: 'Fonction',
        field: 'job',
        cellRenderer: 'jobRenderer',
        cellEditor: 'agSelectCellEditor',
        // Ag select cell editor doesn't accept an object a parameter.
        // Only giving name to display in the select, then recompose the full object when filter on change.
        cellEditorParams: {
          values: this.validatedJobsName,
        },
        onCellValueChanged: this.onJobChange.bind(this),
      },
      {
        headerName: 'Employeur',
        field: 'employer',
        cellRenderer: 'employerRenderer',
        cellEditor: 'agSelectCellEditor',
        cellEditorParams: {
          values: this.validatedEmployersName,
        },
        onCellValueChanged: this.onEmployerChange.bind(this),
      },
      {
        headerName: 'Date de création',
        field: 'createdAt',
        minWidth: 150,
      },
      {
        headerName: 'Actions',
        editable: false,
        minWidth: 150,
        cellRenderer: 'deleteUserComponent',
        cellRendererParams: {
          onClick: this.onDeleteButtonClick.bind(this),
          label: 'Supprimer',
        },
        cellStyle: { 'text-align': 'center' },
      },
    ];
    this.frameworkComponents = {
      deleteUserComponent: DeleteUserComponent,
      jobRenderer: JobRenderer,
      employerRenderer: EmployerRenderer,
      administredStructuresComponent: AdministredStructuresComponent,
    };
    this.findAttachedUsers();
    this.findUnAttachedUsers();
    this.findUnVerifiedUsers();
  }

  public onDeleteButtonClick(arg): void {
    this.deleteUser(arg.data, false);
  }

  public onJobChange(arg): void {
    const selectedJob = this.validatedJobs.find((job) => job.name === arg.newValue);
    if (selectedJob && arg.oldValue === undefined) {
      // user had no job assigned, and admin selected a job -> call setUserJob without modal
      this.adminService.setUserJob(selectedJob._id, arg.data.id).subscribe(() => {
        arg.node.data[arg.colDef.field] = selectedJob;
        arg.api.refreshCells({ rowNodes: [arg.node], columns: [arg.column.colId] });

        this.notificationService.showSuccess('La fonction a bien été attribuée.', '');
      });
      return;
    }
    if (selectedJob === arg.oldValue || selectedJob.name === arg.oldValue.name) {
      arg.node.data[arg.colDef.field] = selectedJob;
      arg.api.refreshCells({ rowNodes: [arg.node], columns: [arg.column.colId] });
      return;
    }
    this.toggleEditJobModal(selectedJob, arg.oldValue, arg);
  }

  public onEmployerChange(arg): void {
    const selectedEmployer = this.validatedEmployers.find((employ) => employ.name === arg.newValue);
    if (selectedEmployer && arg.oldValue === undefined) {
      // user had no employer assigned, and admin selected a employer -> call setUserEmployer
      this.adminService.setUserEmployer(selectedEmployer._id, arg.data.id).subscribe(() => {
        arg.node.data[arg.colDef.field] = selectedEmployer;
        arg.api.refreshCells({ rowNodes: [arg.node], columns: [arg.column.colId] });
        this.notificationService.showSuccess("L'employeur a bien été attribué.", '');
      });
      return;
    }
    if (selectedEmployer === arg.oldValue || selectedEmployer.name === arg.oldValue.name) {
      arg.node.data[arg.colDef.field] = selectedEmployer;
      arg.api.refreshCells({ rowNodes: [arg.node], columns: [arg.column.colId] });
      return;
    }
    this.toggleEditEmployerModal(selectedEmployer, arg.oldValue, arg);
  }

  public deleteUser(user: User, shouldDelete: boolean): void {
    this.toggleDeleteModal(user);
    if (shouldDelete) {
      this.adminService.deleteUser(user._id).subscribe((data) => {
        this.unAttachedUsers = this.unAttachedUsers.filter((obj) => obj._id !== data._id);
        this.unVerifiedUsers = this.unVerifiedUsers.filter((obj) => obj._id !== data._id);
        this.attachedUsers = this.attachedUsers.filter((obj) => obj._id !== data._id);
      });
    }
  }

  public editJob(newJob: Job, oldJob: Job, shouldEdit: boolean, context?): void {
    this.toggleEditJobModal(newJob, oldJob, context);
    if (shouldEdit) {
      // store newJob otherwise cell will only keep name value
      this.adminService.setUserJob(newJob._id, context.node.data._id).subscribe((data) => {
        context.node.data[context.colDef.field] = data.job;
        context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
        this.notificationService.showSuccess("L'opération a réussie.", '');
      });
    } else {
      if (context.oldValue && context.oldValue.name) {
        context.node.data[context.colDef.field] = context.oldValue;
        context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
      }
    }
  }

  public editEmployer(newEmployer: Employer, oldEmployer: Employer, shouldEdit: boolean, context?): void {
    this.toggleEditEmployerModal(newEmployer, oldEmployer, context);
    if (shouldEdit) {
      this.adminService.setUserEmployer(newEmployer._id, context.node.data._id).subscribe((data) => {
        context.node.data[context.colDef.field] = data.employer;
        context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
        this.notificationService.showSuccess("L'opération a réussie.", '');
      });
    } else {
      if (context.oldValue && context.oldValue.name) {
        context.node.data[context.colDef.field] = context.oldValue;
        context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
      }
    }
  }

  public toggleDeleteModal(userToDelete: User): void {
    this.userToDelete = userToDelete;
    this.deleteModalOpenned = !this.deleteModalOpenned;
  }

  public toggleEditJobModal(newValue: Job, oldValue: Job, dataGrid): void {
    this.changingJobs = [newValue, oldValue];
    this.contextRow = dataGrid;
    this.editJobModalOpenned = !this.editJobModalOpenned;
  }

  public toggleEditEmployerModal(newValue: Employer, oldValue: Employer, dataGrid): void {
    this.changingEmployers = [newValue, oldValue];
    this.contextRow = dataGrid;
    this.editEmployerModalOpenned = !this.editEmployerModalOpenned;
  }

  public findValidatedJobs(): void {
    this.adminService.getJobs().subscribe((jobs) => {
      this.validatedJobs = jobs;
      jobs.map((job) => {
        this.validatedJobsName.push(job.name);
      });
    });
  }

  public findValidatedEmployers(): void {
    this.adminService.getEmployers().subscribe((employers) => {
      this.validatedEmployers = employers;
      employers.map((employer) => {
        this.validatedEmployersName.push(employer.name);
      });
    });
  }

  public findAttachedUsers(): void {
    this.adminService.getAttachedUsers().subscribe((users) => {
      this.attachedUsers = users;
      this.attachedUsers.forEach((user) => {
        user._id = user['id'];
      });
    });
  }

  public findUnAttachedUsers(): void {
    this.adminService.getUnAttachedUsers().subscribe((users) => {
      this.unAttachedUsers = users;
      this.unAttachedUsers.forEach((user) => {
        user._id = user['id'];
      });
    });
  }

  public findUnVerifiedUsers(): void {
    this.adminService.getUnVerifiedUsers().subscribe((users) => {
      this.unVerifiedUsers = users;
      this.unVerifiedUsers.forEach((user) => {
        user._id = user['id'];
      });
    });
  }

  public getRowHeight(params): number {
    return params.data.structures ? (params.data.structures.length != 0 ? params.data.structures.length * 40 : 40) : 40;
  }
}
