import { Component, OnInit } from '@angular/core';
import { forkJoin, Observable } from 'rxjs';
import { tap } from 'rxjs/operators';
import { Structure } from '../../models/structure.model';
import { StructureWithOwners } from '../../models/structureWithOwners.model';
import { User } from '../../models/user.model';
import { NotificationService } from '../../services/notification.service';
import { StructureService } from '../../services/structure.service';
import { ButtonType } from '../../shared/components/button/buttonType.enum';
import { ProfileService } from '../services/profile.service';

@Component({
  selector: 'app-structures-management',
  templateUrl: './structures-management.component.html',
  styleUrls: ['./structures-management.component.scss'],
})
export class StructuresManagementComponent implements OnInit {
  public userProfile: User;
  public structures: StructureWithOwners[] = [];
  public selectedStructure: string;
  public deleteInProgress: boolean;
  public buttonTypeEnum = ButtonType;
  public leaveModalOpenned: boolean = false;
  public deleteModalOpenned: boolean = false;
  public cancelDeleteModalOpenned: boolean = false;

  constructor(
    private profileService: ProfileService,
    private structureService: StructureService,
    private notificationService: NotificationService
  ) {}

  ngOnInit(): void {
    this.getUserAndStructures();
  }

  public goBack(): void {
    history.back();
  }

  private getStructures(): void {
    const structures$: Observable<any>[] = [];
    this.structures = [];

    this.userProfile.structuresLink.forEach((structureId) => {
      structures$.push(
        this.structureService.getStructureWithOwners(structureId, null).pipe(
          tap((structure) => {
            this.structures.push(structure);
          })
        )
      );
    });
    forkJoin(structures$).subscribe(() => {
      this.structures.sort((a, b) => a.structure.structureName.localeCompare(b.structure.structureName));
    });
  }

  private getUserAndStructures(): void {
    this.profileService.getProfile().then((profile: User) => {
      this.userProfile = profile;
      this.getStructures();
    });
  }

  public leaveStructure(structure: Structure, shouldExclude: boolean): void {
    this.leaveModalOpenned = false;
    if (shouldExclude) {
      this.structureService.removeOwnerFromStructure(this.userProfile._id, structure._id).subscribe(
        () => {
          this.structures = this.structures.filter((obj) => obj.structure._id !== structure._id);
          this.notificationService.showSuccess(`Vous avez bien quitté ${structure.structureName}`, '');
        },
        () => {
          this.notificationService.showError('Une erreur est survenue, veuillez réessayer.', '');
        }
      );
    }
  }

  public deleteStructure(structure: Structure, shouldDelete: boolean): void {
    this.deleteModalOpenned = false;
    if (shouldDelete) {
      this.structureService.delete(structure._id).subscribe((res) => {
        this.getUserAndStructures();
      });
    }
  }

  public isBeingDeleted(structure: Structure): boolean {
    return Boolean(structure.toBeDeletedAt);
  }

  public cancelDelete(structure: Structure, shouldCancel: boolean): void {
    this.cancelDeleteModalOpenned = false;
    if (shouldCancel) {
      this.structureService.cancelDelete(structure._id).subscribe((res) => {
        this.getStructures();
      });
    }
  }
}
